import { DeclarationReflection, type Type, type TypeParameterReflection } from "typedoc";
/** Type for specifying the style of a text. */
type TextStyle = {
    font: {
        family: string;
        size: number;
        bold: boolean;
        italic: boolean;
        underline: boolean;
        strikeout: boolean;
    };
    color: string;
    backgroundColor: string;
};
/**
 * Options for the PlantUmlGenerator.
 */
export type PlantUmlCodeGeneratorOptions = {
    type: "none" | "simple" | "detailed";
    methodParameterOutput: "none" | "only-names" | "only-types" | "complete";
    memberOrder: "abc" | "public-to-private" | "private-to-public";
    topDownLayoutMaxSiblings: number;
    visibilityStyle: "text" | "icon";
    hideEmptyMembers: boolean;
    hideCircledChar: boolean;
    hideShadow: boolean;
    diagramBackgroundColor: string;
    boxBackgroundColor: string;
    boxBorderWidth: number;
    boxBorderColor: string;
    boxBorderRadius: number;
    arrowWidth: number;
    arrowColor: string;
    className: TextStyle;
    interfaceName: TextStyle;
    propertyName: TextStyle;
    propertyType: TextStyle;
    methodName: TextStyle;
    methodParameterName: TextStyle;
    methodParameterType: TextStyle;
    methodReturnType: TextStyle;
};
/**
 * Class that generates PlantUML code.
 */
export declare class PlantUmlCodeGenerator {
    /** The options for the PlantUML output. */
    private readonly options;
    /** Compare function used for sorting class members. */
    private readonly classMemberCompareFunction;
    /**
     * Creates a new PlantUmlGenerator object with the given options.
     * @param options The options for the PlantUML output.
     */
    constructor(options: Readonly<PlantUmlCodeGeneratorOptions>);
    /**
     * Generates the Plant UML lines for the class diagram of the given reflection.
     * @param reflection The reflection for which to generate a class diagram.
     * @returns The Plant UML lines for the class diagram of the given reflection.
     *          If the given reflection is not part of an inheritance or implementation, the result is an empty array.
     */
    createClassDiagramPlantUmlForReflection(reflection: Readonly<DeclarationReflection>): string[];
    /**
     * Creates an array of PlantUML lines for generating the box (including its properties and methods) of a reflection.
     * @param reflection The reflection for which the PlantUML should be generated.
     * @param typeArguments Possible type arguments if the reflection is based on a template class.
     * @param isType Specifies whether the reflection belongs to a type. This parameter is necessary to distinguish
     *               between a template class and a type based on the template class, because missing typeArguments
     *               can either mean that the default types for all type parameters are used for the type or that the
     *               reflection stands for a template class.
     * @returns The PlantUML lines for the reflection.
     */
    protected createPlantUmlForReflection(reflection: Readonly<DeclarationReflection>, typeArguments?: ReadonlyArray<Type>, isType?: boolean): string[];
    /**
     * Returns a list of PlantUML commands to format the diagram using the current options.
     * @param siblingsAbove The number of siblings above the element the diagram is generated for.
     * @param siblingsBelow The number of siblings below the element the diagram is generated for.
     * @returns The PlantUML commands to format the diagram.
     */
    private getDiagramFormatCommands;
    /**
     * Returns a list of PlantUML commands to format the boxes using the current options.
     * @returns The PlantUML commands to format the boxes.
     */
    private getBoxFormatCommands;
    /**
     * Returns a list of PlantUML commands to format the class names using the current options.
     * @returns The PlantUML commands to format the class names.
     */
    private getClassNameFormatCommands;
    /**
     * Returns a list of PlantUML commands to format the interface names using the current options.
     * @returns The PlantUML commands to format the interface names.
     */
    private getInterfaceNameFormatCommands;
    /**
     * Creates an array of PlantUML lines for generating the box (including its properties and methods) of a type.
     * @param type The type for which the PlantUML should be generated.
     * @returns The PlantUML lines for the type.
     */
    private createPlantUmlForType;
    /**
     * Returns the PlantUML line for generating the output for a given method.
     * @param methodFlags Flags for the method the signature belongs to.
     * @param signature Data about the method signature.
     * @param typeParamsMap If the method belongs to a class which has type arguments this map contains the mapping of
     *                      the type parameters to their type arguments.
     * @returns The PlantUML line for the given method.
     */
    private createPlantUmlForMethodSignature;
    /**
     * Returns the PlantUML line for generating the output for a given property.
     * @param property The property for which the PlantUML should be generated.
     * @param typeParamsMap If the property belongs to a class which has type arguments this map contains the mapping of
     *                      the type parameters to their type arguments.
     * @returns The PlantUML line for the given property.
     */
    private createPlantUmlForProperty;
    /**
     * Creates a map which includes the type argument names for every type parameter name.
     * @param typeParameters The type parameters.
     * @param typeArguments The possible type arguments.
     * @returns A map which includes the type argument names for every type parameter name.
     */
    protected static createTypeParameterMapping(typeParameters: ReadonlyArray<TypeParameterReflection>, typeArguments?: ReadonlyArray<Type>): Map<string, string>;
    /**
     * Escapes the name by putting double quotes around it preventing a PlantUML syntax error.
     * @param name The original name.
     * @returns The escaped name.
     */
    private static escapeName;
    /**
     * Returns the name of a type. If the type includes template parameters those are replaced with their values
     * using the second argument (typeParamsMap).
     * @param type The type whos name is wanted.
     * @param typeParamsMap Possible template parameter values.
     * @returns The name of the type.
     */
    private static getTypeNameWithReplacedTypeParameters;
    /**
     * Returns the full name of a given reflection. If the reflection is a template class and therefore has type
     * parameters, these are included in the name as <A, B, C>. If the reflection is based on a template class and
     * therefore has type arguments, these are included in the name as <string, number, any>.
     * @param reflection The reflection whos full name is wanted.
     * @param typeParamsMap If the property belongs to a class which has type arguments this map contains the mapping of
     *                      the type parameters to their type arguments.
     * @returns The full name of the reflection.
     */
    private static getFullReflectionName;
    /**
     * Returns the PlantUML line for the introduction of a class or interface.
     * @param reflection The class or interface for which the PlantUML should be generated.
     * @param typeParamsMap If the property belongs to a class which has type arguments this map contains the mapping of
     *                      the type parameters to their type arguments.
     * @returns The PlantUML line for the given class or interface.
     */
    private static createPlantUmlForClassOrInterface;
    /**
     * Returns the full name of a given type. If the type has a reflection, the full name of that reflection is
     * returned. If the type is based on a template class and therefore has type arguments, these are included
     * in the name as <D, E, F>.
     * @param type The type whos full name is wanted.
     * @returns The full name of the type.
     */
    private static getFullTypeName;
    /**
     * Creates dummy PlantUML lines for the given type.
     * @param type The type for which the PlantUML lines are generated.
     * @returns The PlantUML lines for the given type.
     */
    private static createDummyPlantUmlForType;
    /**
     * Adds legacy HTML to the given text to format it according to the given style.
     * @param text The text that should be formatted.
     * @param style The style that should be used when formating the text.
     * @returns The text extended with legacy HTML to format it.
     */
    private static formatText;
}
export {};
