var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { PlantUmlPipe } from "plantuml-pipe";
import Queue from "queue-fifo";
/**
 * Class for generating diagrams from PlantUML code.
 * This class can spawn multiple PlantUML processes and evenly schedules the work between them.
 */
export class PlantUmlDiagramGenerator {
    /**
     * Creates a new PlantUML generator object.
     * @param processCount Number of PlantUML processes used to schedule the diagram generation.
     * @param format The format for the generated diagrams.
     */
    constructor(processCount, format) {
        /** The object which encapsulates the PlantUML processes. */
        this.plantUmlProcesses = new Array();
        /** Index of the process that should generate the next diagram. */
        this.nextProcessToGetWorkIndex = 0;
        if (processCount <= 0) {
            throw new Error("processCount must be > 0");
        }
        this.processCount = processCount;
        this.outputFormat = format;
    }
    /**
     * Writes the PlantUML code to a PlantUML process that generates the diagram.
     * @param plantUml The PlantUML code for the diagram.
     * @returns A promise that the diagram has been generated.
     */
    generate(plantUml) {
        return __awaiter(this, void 0, void 0, function* () {
            const process = this.getNextProcess();
            // Here we create a promise that is resolved once the diagram has been generated
            let resolver;
            const promise = new Promise((resolve) => {
                resolver = resolve;
            });
            process.diagramQueue.enqueue({
                resolve: resolver !== null && resolver !== void 0 ? resolver : (() => { }),
            });
            process.pipe.in.write(plantUml);
            return promise;
        });
    }
    /**
     * Notifies the PlantUML diagram generator processes that there is no more job to do.
     */
    shutdown() {
        for (const process of this.plantUmlProcesses) {
            process.pipe.in.end();
        }
    }
    /**
     * Returns an object which encapsulates a PlantUML process that should generate the next diagram.
     * @returns An object for the process that should generate the next diagram.
     */
    getNextProcess() {
        const processToWorkIndex = this.nextProcessToGetWorkIndex;
        this.maybeSpawnNewProcess();
        this.setNextProcessForWork();
        return this.plantUmlProcesses[processToWorkIndex];
    }
    /**
     * If necessary, spawns a new PlantUML process and stores it in the internal process array.
     */
    maybeSpawnNewProcess() {
        if (this.nextProcessToGetWorkIndex >= this.plantUmlProcesses.length) {
            const newPipe = new PlantUmlPipe({ outputFormat: this.outputFormat });
            const newQueue = new Queue();
            newPipe.out.on("data", (imageData) => {
                const diagram = newQueue.dequeue();
                diagram.resolve(imageData);
            });
            this.plantUmlProcesses.push({ pipe: newPipe, diagramQueue: newQueue });
        }
    }
    /**
     * Sets the internal index for the process that should receive the next job.
     */
    setNextProcessForWork() {
        this.nextProcessToGetWorkIndex++;
        if (this.nextProcessToGetWorkIndex === this.processCount) {
            this.nextProcessToGetWorkIndex = 0;
        }
    }
}
