import { type Application, type Context, PageEvent, RendererEvent } from "typedoc";
/**
 * The UML class diagram generator plugin.
 *
 * # What does it do?
 *
 * This plugin generates an UML class diagram for classes and interfaces.
 *
 * # How does it do it?
 *
 * 1. The plugin scans through the classes and interfaces and generates PlantUML code for them.
 * 2. The plugin uses the plantuml-pipe module to generate image files from the PlantUML code.
 * 3. The plugin adds a section with the image to the page of each class and interface.
 */
export declare class Plugin {
    /** The TypeDoc application instance that loaded the plugin. */
    private typedoc;
    /** The options of this plugin. */
    private readonly options;
    /** The directory in which the plugin generates files. */
    private outputDirectory;
    /** Stores how many diagrams the plugin has to generate. */
    private numberOfDiagramsToGenerate;
    /** Object that uses TypeDoc reflection data to generate PlantUML code. */
    private plantUmlCodeGenerator;
    /** Object that uses PlantUML code to generate PlantUML diagrams. */
    private plantUmlDiagramGenerator;
    /** Progress bar shown when generating the diagrams. */
    private progressBar;
    /** Logger for verbose output in debug mode. */
    private log;
    /** Stores the legends for the diagram of every reflection (KEY = ID of the reflection). */
    private readonly diagramLegends;
    /**
     * Checks if the plugin is active and should generate output.
     * @returns True if the plugin is active, otherwise false.
     */
    get isActive(): boolean;
    /**
     * Checks if the plugin has anything to do.
     * @returns True if the plugin needs to generate diagrams, otherwise false.
     */
    get hasWork(): boolean;
    /**
     * Checks if the plugin is generating images from the generated PlantUML code.
     * @returns True if the plugin is generating images, otherwise false.
     */
    get isGeneratingImages(): boolean;
    /**
     * Checks if the plugin should generate diagram legends.
     * @returns True if the plugin should generate diagram legends, otherwise false.
     */
    get shouldGenerateLegends(): boolean;
    /**
     * Initializes the plugin.
     * @param typedoc The TypeDoc application.
     */
    initialize(typedoc: Application): void;
    /**
     * Triggered when the TypeDoc converter begins resolving a project.
     * Reads plugin parameter values.
     * @param context Describes the current state the converter is in.
     */
    onConverterResolveBegin(context: Context): void;
    /**
     * Triggered when the TypeDoc converter has finished resolving a project.
     * @param context Describes the current state the converter is in.
     */
    onConverterResolveEnd(context: Context): void;
    /**
     * Triggered before the renderer starts rendering a project.
     * Setup image generator output directory and progress bar.
     * @param event The event emitted by the renderer class.
     */
    onRendererBegin(event: RendererEvent): void;
    /**
     * Triggered after a document has been rendered, just before it is written to disc.
     * Generates a UML class diagram and adds it to the page of the reflection.
     * @param event The event emitted by the renderer class.
     */
    onRendererEndPage(event: PageEvent): void;
    /**
     * Triggered after the renderer has written all documents.
     * Appends style data to the main CSS file.
     * @param event The event emitted by the renderer class.
     */
    onRendererEnd(event: RendererEvent): void;
    /**
     * Adds the plugin's options to the application's options.
     */
    private addOptionsToApplication;
    /**
     * Subscribes to events of the application so that the plugin can do its work
     * in the particular doc generation phases.
     */
    private subscribeToApplicationEvents;
    /**
     * Creates the PlantUML code generator options using the current plugin options.
     * @returns The options for the PlantUML code generator.
     */
    private createPlantUmlCodeGeneratorOptions;
    /**
     * Computes how many diagrams the plugin must generate.
     * @param project The project containing the reflections for which the diagrams are generated.
     * @returns The number of diagrams to generate.
     */
    private computeDiagramCount;
    /**
     * Checks if the plugin should process a given reflection.
     * @param reflection The reflection in question.
     * @returns True, if the plugin should process the reflection, otherwise false.
     */
    private shouldProcessReflection;
    /**
     * Checks if the plugin should process a given page.
     * @param pageContent The content of the page.
     * @returns True, if the plugin should process the page, otherwise false.
     */
    private shouldProcessPage;
    /**
     * Processes the page by inserting a hierarchy diagram into it.
     * @param event The page event with the page data.
     */
    private processPage;
    /**
     * Processes the image data of a generated diagram by creating an image URL and adding it to the reflection's page.
     * @param reflection The reflection for which the image was generated.
     * @param pageFilePath Path to the output file of the reflection's doc page.
     * @param imageData The data of the generated image.
     */
    private readonly processImageData;
    /**
     * Creates a local file with the class diagram of a reflection.
     * @param imageData The image data of the class diagram.
     * @param reflection The reflection for which the file is written.
     * @returns The absolute path to the file that was created.
     */
    private writeLocalImageFileForReflection;
    /**
     * Creates a file containing the PlantUML code.
     * @param plantUml The PlantUML code that should be written into the file.
     * @param reflection The reflection for which the file is written.
     */
    private writePlantUmlFileForReflection;
    /**
     * Creates a legend for the reflection and stores it in the internal map.
     * @param reflection The reflection for which the legend should be generated.
     * @param plantUmlLines The PlantUML lines for the reflection's diagram.
     */
    private createLegendForReflection;
    /**
     * Inserts the hierarchy diagram into the page file.
     * @param filePath Absolute path to the page file.
     * @param reflection The reflection for which the diagram was generated.
     * @param imageUrl The URL to the image of the diagram.
     */
    private insertHierarchyDiagramIntoFile;
    /**
     * Inserts the hierarchy diagram into the content string.
     * @param originalContent The content string into which the diagram is inserted.
     * @param reflection The reflection for which the diagram was generated.
     * @param imageUrl The URL to the image of the diagram.
     * @returns The updated content string with the diagram inserted into it.
     */
    private insertHierarchyDiagramIntoContent;
}
