var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as fs from "fs";
import * as path from "path";
import ProgressBar from "progress";
import { Converter, DeclarationReflection, PageEvent, ReflectionKind, RendererEvent, } from "typedoc";
import { createDiagramLegendForPlantUml, DiagramLegend } from "./diagram_legend.js";
import { createEmbeddedImageUrl, createLocalImageFileUrl, createRemoteImageUrl } from "./image_url_generator.js";
import { Logger } from "./logger.js";
import { CachingPlantUmlCodeGenerator } from "./plantuml/caching_plantuml_code_generator.js";
import {} from "./plantuml/plantuml_code_generator.js";
import { PlantUmlDiagramGenerator } from "./plantuml/plantuml_diagram_generator.js";
import { PluginOptions } from "./plugin_options.js";
import { PageProcessor } from "./typedoc/page_processor.js";
import { createHierarchyDiagramSection, PageSections } from "./typedoc/page_section.js";
import { PageSectionFinder } from "./typedoc/page_section_finder.js";
import { reflectionIsPartOfClassHierarchy } from "./typedoc/typedoc_utils.js";
/**
 * The UML class diagram generator plugin.
 *
 * # What does it do?
 *
 * This plugin generates an UML class diagram for classes and interfaces.
 *
 * # How does it do it?
 *
 * 1. The plugin scans through the classes and interfaces and generates PlantUML code for them.
 * 2. The plugin uses the plantuml-pipe module to generate image files from the PlantUML code.
 * 3. The plugin adds a section with the image to the page of each class and interface.
 */
export class Plugin {
    constructor() {
        /** The options of this plugin. */
        this.options = new PluginOptions();
        /** Stores how many diagrams the plugin has to generate. */
        this.numberOfDiagramsToGenerate = 0;
        /** Stores the legends for the diagram of every reflection (KEY = ID of the reflection). */
        this.diagramLegends = new Map();
        /**
         * Processes the image data of a generated diagram by creating an image URL and adding it to the reflection's page.
         * @param reflection The reflection for which the image was generated.
         * @param pageFilePath Path to the output file of the reflection's doc page.
         * @param imageData The data of the generated image.
         */
        this.processImageData = (reflection, pageFilePath, imageData) => {
            var _a, _b, _c, _d, _e;
            let imageUrl = "";
            if (this.options.location === "local") {
                (_a = this.log) === null || _a === void 0 ? void 0 : _a.info(`Writing local image file for reflection ${reflection.name} ...`);
                const absoluteImageFilePath = this.writeLocalImageFileForReflection(imageData, reflection);
                (_b = this.log) === null || _b === void 0 ? void 0 : _b.info(`Creating local image URL for reflection ${reflection.name} ...`);
                imageUrl = createLocalImageFileUrl(pageFilePath, absoluteImageFilePath);
            }
            else {
                (_c = this.log) === null || _c === void 0 ? void 0 : _c.info(`Creating embedded image URL for reflection ${reflection.name} ...`);
                imageUrl = createEmbeddedImageUrl(imageData, this.options.format);
            }
            (_d = this.log) === null || _d === void 0 ? void 0 : _d.info(`Inserting diagram into page for reflection ${reflection.name} ...`);
            this.insertHierarchyDiagramIntoFile(pageFilePath, reflection, imageUrl);
            (_e = this.progressBar) === null || _e === void 0 ? void 0 : _e.tick();
        };
    }
    /**
     * Checks if the plugin is active and should generate output.
     * @returns True if the plugin is active, otherwise false.
     */
    get isActive() {
        return this.options.type === "simple" || this.options.type === "detailed";
    }
    /**
     * Checks if the plugin has anything to do.
     * @returns True if the plugin needs to generate diagrams, otherwise false.
     */
    get hasWork() {
        return this.numberOfDiagramsToGenerate > 0;
    }
    /**
     * Checks if the plugin is generating images from the generated PlantUML code.
     * @returns True if the plugin is generating images, otherwise false.
     */
    get isGeneratingImages() {
        return this.options.location === "local" || this.options.location === "embed";
    }
    /**
     * Checks if the plugin should generate diagram legends.
     * @returns True if the plugin should generate diagram legends, otherwise false.
     */
    get shouldGenerateLegends() {
        // The simple diagram type can only contain the circled chars.
        // If those are deactivated then it does not make sense to have any legends.
        if (this.options.type === "simple" && this.options.hideCircledChar) {
            return false;
        }
        else if (this.options.legendType === "only-included" || this.options.legendType === "full") {
            return true;
        }
        return false;
    }
    /**
     * Initializes the plugin.
     * @param typedoc The TypeDoc application.
     */
    initialize(typedoc) {
        this.typedoc = typedoc;
        this.addOptionsToApplication();
        this.subscribeToApplicationEvents();
    }
    /**
     * Triggered when the TypeDoc converter begins resolving a project.
     * Reads plugin parameter values.
     * @param context Describes the current state the converter is in.
     */
    onConverterResolveBegin(context) {
        this.options.readValuesFromApplication(context.converter.owner);
    }
    /**
     * Triggered when the TypeDoc converter has finished resolving a project.
     * @param context Describes the current state the converter is in.
     */
    onConverterResolveEnd(context) {
        var _a, _b, _c;
        if (this.isActive) {
            if (this.options.createVerboseOutput) {
                this.log = new Logger();
            }
            (_a = this.log) === null || _a === void 0 ? void 0 : _a.info("Caculating number of diagrams to generate ...");
            this.numberOfDiagramsToGenerate = this.computeDiagramCount(context.project);
            (_b = this.log) === null || _b === void 0 ? void 0 : _b.info("The result is: " + this.numberOfDiagramsToGenerate.toString());
            if (this.hasWork) {
                this.plantUmlCodeGenerator = new CachingPlantUmlCodeGenerator(this.createPlantUmlCodeGeneratorOptions());
                if (this.isGeneratingImages) {
                    this.plantUmlDiagramGenerator = new PlantUmlDiagramGenerator(this.options.generatorProcessCount, this.options.format);
                    if (!this.options.hideProgressBar) {
                        (_c = this.log) === null || _c === void 0 ? void 0 : _c.info("Setting up progress bar ...");
                        this.progressBar = new ProgressBar(
                        // eslint-disable-next-line max-len
                        `Using ${this.options.generatorProcessCount} process(es) to add ${this.numberOfDiagramsToGenerate} class diagrams [:bar] :percent`, {
                            total: this.numberOfDiagramsToGenerate,
                            width: 40,
                        });
                    }
                }
            }
        }
    }
    /**
     * Triggered before the renderer starts rendering a project.
     * Setup image generator output directory and progress bar.
     * @param event The event emitted by the renderer class.
     */
    onRendererBegin(event) {
        if (this.isActive && this.hasWork) {
            this.outputDirectory = path.join(event.outputDirectory, "assets/");
        }
    }
    /**
     * Triggered after a document has been rendered, just before it is written to disc.
     * Generates a UML class diagram and adds it to the page of the reflection.
     * @param event The event emitted by the renderer class.
     */
    onRendererEndPage(event) {
        var _a, _b;
        if (this.isActive && this.hasWork) {
            if (this.shouldProcessReflection(event.model)) {
                if (this.shouldProcessPage(event.contents)) {
                    (_a = this.log) === null || _a === void 0 ? void 0 : _a.info(`Processing page for reflection ${event.model.name} ...`);
                    this.processPage(event);
                }
                else {
                    (_b = this.log) === null || _b === void 0 ? void 0 : _b.info(`Skipping page for reflection ${event.model.name} ...`);
                }
            }
        }
    }
    /**
     * Triggered after the renderer has written all documents.
     * Appends style data to the main CSS file.
     * @param event The event emitted by the renderer class.
     */
    onRendererEnd(event) {
        var _a, _b;
        if (this.isActive && this.hasWork) {
            (_a = this.log) === null || _a === void 0 ? void 0 : _a.info("Attaching content to CSS file ...");
            const filename = path.join(event.outputDirectory, "assets/style.css");
            let data = fs.readFileSync(filename, "utf8") +
                // eslint-disable-next-line max-len
                "\n.tsd-hierarchy-diagram .diagram { max-width: 100%; display: block; margin: 0 auto; text-align: center; }\n";
            if (this.shouldGenerateLegends) {
                data += DiagramLegend.getCss();
            }
            fs.writeFileSync(filename, data, "utf8");
            (_b = this.log) === null || _b === void 0 ? void 0 : _b.info("DONE");
            if (this.isGeneratingImages && this.plantUmlDiagramGenerator) {
                this.plantUmlDiagramGenerator.shutdown();
            }
        }
    }
    /**
     * Adds the plugin's options to the application's options.
     */
    addOptionsToApplication() {
        this.options.addToApplication(this.typedoc);
    }
    /**
     * Subscribes to events of the application so that the plugin can do its work
     * in the particular doc generation phases.
     */
    subscribeToApplicationEvents() {
        this.typedoc.converter.on(Converter.EVENT_RESOLVE_BEGIN, (c) => {
            this.onConverterResolveBegin(c);
        });
        this.typedoc.converter.on(Converter.EVENT_RESOLVE_END, (c) => {
            this.onConverterResolveEnd(c);
        });
        this.typedoc.renderer.on(RendererEvent.BEGIN, (e) => {
            this.onRendererBegin(e);
        });
        this.typedoc.renderer.on(PageEvent.END, (e) => {
            this.onRendererEndPage(e);
        });
        this.typedoc.renderer.on(RendererEvent.END, (e) => {
            this.onRendererEnd(e);
        });
    }
    /**
     * Creates the PlantUML code generator options using the current plugin options.
     * @returns The options for the PlantUML code generator.
     */
    createPlantUmlCodeGeneratorOptions() {
        return {
            type: this.options.type,
            methodParameterOutput: this.options.methodParameterOutput,
            memberOrder: this.options.memberOrder,
            topDownLayoutMaxSiblings: this.options.topDownLayoutMaxSiblings,
            visibilityStyle: this.options.visibilityStyle,
            hideEmptyMembers: this.options.hideEmptyMembers,
            hideCircledChar: this.options.hideCircledChar,
            hideShadow: this.options.hideShadow,
            diagramBackgroundColor: this.options.backgroundColor,
            boxBackgroundColor: this.options.box.backgroundColor,
            boxBorderWidth: this.options.box.border.width,
            boxBorderColor: this.options.box.border.color,
            boxBorderRadius: this.options.box.border.radius,
            arrowWidth: this.options.arrow.width,
            arrowColor: this.options.arrow.color,
            className: this.options.class.name,
            interfaceName: this.options.interface.name,
            propertyName: this.options.property.name,
            propertyType: this.options.property.type,
            methodName: this.options.method.name,
            methodParameterName: this.options.method.parameter.name,
            methodParameterType: this.options.method.parameter.type,
            methodReturnType: this.options.method.returnType,
        };
    }
    /**
     * Computes how many diagrams the plugin must generate.
     * @param project The project containing the reflections for which the diagrams are generated.
     * @returns The number of diagrams to generate.
     */
    computeDiagramCount(project) {
        let count = 0;
        for (const key in project.reflections) {
            const reflection = project.reflections[key];
            if (this.shouldProcessReflection(reflection)) {
                ++count;
            }
        }
        return count;
    }
    /**
     * Checks if the plugin should process a given reflection.
     * @param reflection The reflection in question.
     * @returns True, if the plugin should process the reflection, otherwise false.
     */
    // eslint-disable-next-line class-methods-use-this
    shouldProcessReflection(reflection) {
        if (reflection instanceof DeclarationReflection &&
            (reflection.kind === ReflectionKind.Class || reflection.kind === ReflectionKind.Interface) &&
            reflectionIsPartOfClassHierarchy(reflection)) {
            return true;
        }
        return false;
    }
    /**
     * Checks if the plugin should process a given page.
     * @param pageContent The content of the page.
     * @returns True, if the plugin should process the page, otherwise false.
     */
    // eslint-disable-next-line class-methods-use-this
    shouldProcessPage(pageContent) {
        if (pageContent && PageSectionFinder.hasSection(pageContent, PageSections.Hierarchy)) {
            return true;
        }
        return false;
    }
    /**
     * Processes the page by inserting a hierarchy diagram into it.
     * @param event The page event with the page data.
     */
    processPage(event) {
        var _a, _b, _c, _d, _e, _f;
        const reflection = event.model;
        (_a = this.log) === null || _a === void 0 ? void 0 : _a.info(`Generating PlantUML code for reflection ${reflection.name} ...`);
        const plantUmlLines = this.plantUmlCodeGenerator.createClassDiagramPlantUmlForReflection(reflection);
        const plantUml = plantUmlLines.join("\n");
        if (this.options.createPlantUmlFiles) {
            (_b = this.log) === null || _b === void 0 ? void 0 : _b.info(`Writing PlantUML file for reflection ${reflection.name} ...`);
            this.writePlantUmlFileForReflection(plantUml, reflection);
        }
        if (this.shouldGenerateLegends) {
            (_c = this.log) === null || _c === void 0 ? void 0 : _c.info(`Creating leged for diagram of reflection ${reflection.name} ...`);
            this.createLegendForReflection(reflection, plantUmlLines);
        }
        if (this.isGeneratingImages && this.plantUmlDiagramGenerator) {
            (_d = this.log) === null || _d === void 0 ? void 0 : _d.info(`Creating diagram image for reflection ${reflection.name} ...`);
            const job = this.plantUmlDiagramGenerator.generate(plantUml + "\n").then((imageData) => {
                this.processImageData(reflection, event.filename, imageData);
            });
            // Notify TypeDoc of the async job so that it can await it before shutting itself down.
            // Without this the generator processes are killed when the TypeDoc process ends.
            this.typedoc.renderer.postRenderAsyncJobs.push(() => __awaiter(this, void 0, void 0, function* () { return job; }));
        }
        else if (this.options.location === "remote") {
            (_e = this.log) === null || _e === void 0 ? void 0 : _e.info(`Creating remote image URL for reflection ${reflection.name} ...`);
            const imageUrl = createRemoteImageUrl(this.options.remoteBaseUrl, plantUml, this.options.format);
            (_f = this.log) === null || _f === void 0 ? void 0 : _f.info(`Inserting diagram into page of reflection ${reflection.name} ...`);
            event.contents = this.insertHierarchyDiagramIntoContent(event.contents, reflection, imageUrl);
        }
    }
    /**
     * Creates a local file with the class diagram of a reflection.
     * @param imageData The image data of the class diagram.
     * @param reflection The reflection for which the file is written.
     * @returns The absolute path to the file that was created.
     */
    writeLocalImageFileForReflection(imageData, reflection) {
        const filename = `${reflection.name}-umlClassDiagram-${reflection.id}.${this.options.format}`;
        const absoluteFilePath = path.join(this.outputDirectory, filename);
        fs.writeFileSync(absoluteFilePath, imageData);
        return absoluteFilePath;
    }
    /**
     * Creates a file containing the PlantUML code.
     * @param plantUml The PlantUML code that should be written into the file.
     * @param reflection The reflection for which the file is written.
     */
    writePlantUmlFileForReflection(plantUml, reflection) {
        const filename = `${reflection.name}-umlClassDiagram-${reflection.id}.puml`;
        const absoluteFilePath = path.join(this.outputDirectory, filename);
        fs.writeFileSync(absoluteFilePath, plantUml, "utf8");
    }
    /**
     * Creates a legend for the reflection and stores it in the internal map.
     * @param reflection The reflection for which the legend should be generated.
     * @param plantUmlLines The PlantUML lines for the reflection's diagram.
     */
    createLegendForReflection(reflection, plantUmlLines) {
        const legend = this.options.legendType === "only-included"
            ? createDiagramLegendForPlantUml(plantUmlLines)
            : new DiagramLegend();
        if (this.options.hideCircledChar) {
            legend.hideTypeIcons();
        }
        if (this.options.isAnythingStyledItalic) {
            // PlantUML uses an italic font for abstract members. If the user specified an italic font for anything
            // then this is no longer reasonable and we hide this entry in the legend to avoid confusion.
            legend.hideAbstractMemberItem();
        }
        this.diagramLegends.set(reflection.id, legend);
    }
    /**
     * Inserts the hierarchy diagram into the page file.
     * @param filePath Absolute path to the page file.
     * @param reflection The reflection for which the diagram was generated.
     * @param imageUrl The URL to the image of the diagram.
     */
    insertHierarchyDiagramIntoFile(filePath, reflection, imageUrl) {
        let fileContent = fs.readFileSync(filePath, "utf8");
        fileContent = this.insertHierarchyDiagramIntoContent(fileContent, reflection, imageUrl);
        fs.writeFileSync(filePath, fileContent, "utf8");
    }
    /**
     * Inserts the hierarchy diagram into the content string.
     * @param originalContent The content string into which the diagram is inserted.
     * @param reflection The reflection for which the diagram was generated.
     * @param imageUrl The URL to the image of the diagram.
     * @returns The updated content string with the diagram inserted into it.
     */
    insertHierarchyDiagramIntoContent(originalContent, reflection, imageUrl) {
        const page = new PageProcessor(originalContent);
        const legend = this.diagramLegends.get(reflection.id);
        const hierarchyDiagramSection = createHierarchyDiagramSection(this.options.sectionTitle, imageUrl, reflection.name, legend && !legend.isEmpty ? legend.getHtml(this.options.visibilityStyle) : "");
        if (this.options.position === "above") {
            page.insertAboveSection(PageSections.Hierarchy, hierarchyDiagramSection);
        }
        else {
            page.insertBelowSection(PageSections.Hierarchy, hierarchyDiagramSection);
        }
        return page.content;
    }
}
