import { type Application } from "typedoc";
type DeepRequired<T> = {
    [K in keyof T]: DeepRequired<T[K]>;
} & Required<T>;
/** Type for specifying the style of a text. */
type TextStyle = {
    font?: {
        family?: string;
        size?: number;
        bold?: boolean;
        italic?: boolean;
        underline?: boolean;
        strikeout?: boolean;
    };
    color?: string;
    backgroundColor?: string;
};
/**
 * Type defining all options and their possible values of the plugin.
 */
export type PluginConfig = {
    type?: "none" | "simple" | "detailed";
    location?: "local" | "remote" | "embed";
    remoteBaseUrl?: string;
    format?: "png" | "svg";
    sectionTitle?: string;
    position?: "above" | "below";
    legendType?: "none" | "only-included" | "full";
    methodParameterOutput?: "none" | "only-names" | "only-types" | "complete";
    memberOrder?: "abc" | "public-to-private" | "private-to-public";
    topDownLayoutMaxSiblings?: number;
    visibilityStyle?: "text" | "icon";
    hideEmptyMembers?: boolean;
    hideCircledChar?: boolean;
    hideShadow?: boolean;
    style?: {
        backgroundColor?: string;
        box?: {
            backgroundColor?: string;
            border?: {
                width?: number;
                color?: string;
                radius?: number;
            };
        };
        arrow?: {
            width?: number;
            color?: string;
        };
        text?: TextStyle;
        class?: {
            name?: TextStyle;
        };
        interface?: {
            name?: TextStyle;
        };
        property?: {
            name?: TextStyle;
            type?: TextStyle;
        };
        method?: {
            name?: TextStyle;
            parameter?: {
                name?: TextStyle;
                type?: TextStyle;
            };
            returnType?: TextStyle;
        };
    };
    generatorProcessCount?: number;
    hideProgressBar?: boolean;
    createPlantUmlFiles?: boolean;
    verboseOutput?: boolean;
};
/** Plugin options type without nullables. */
type RequiredPluginConfig = DeepRequired<PluginConfig>;
/**
 * Extend typedoc's options with the plugin's option using declaration merging.
 */
declare module "typedoc" {
    interface TypeDocOptionMap {
        umlClassDiagram: PluginConfig;
    }
}
/**
 * Class storing the options of the plugin.
 */
export declare class PluginOptions {
    /** Plugin option values as provided by the user. */
    private userValues;
    /**
     * Returns how detailed the generated class diagrams should be.
     * @returns How detailed the generated class diagrams should be.
     */
    get type(): RequiredPluginConfig["type"];
    /**
     * Returns the location where the class diagrams should be stored.
     * @returns The location where the class diagrams should be stored.
     */
    get location(): RequiredPluginConfig["location"];
    /**
     * Returns the base URL used when generating remote images.
     * @returns The base URL used when generating remote images.
     */
    get remoteBaseUrl(): RequiredPluginConfig["remoteBaseUrl"];
    /**
     * Returns the image format used for the class diagrams.
     * @returns The image format used for the class diagrams.
     */
    get format(): RequiredPluginConfig["format"];
    /**
     * Returns the title that should be used for the section that contains the class diagrams.
     * @returns The title that should be used for the section that contains the class diagrams.
     */
    get sectionTitle(): RequiredPluginConfig["sectionTitle"];
    /**
     * Returns where on the page the class diagrams should be put.
     * @returns Where on the page the class diagrams should be put.
     */
    get position(): RequiredPluginConfig["position"];
    /**
     * Returns the type of legend that should be generated for the class diagrams.
     * @returns The type of legend that should be generated for the class diagrams.
     */
    get legendType(): RequiredPluginConfig["legendType"];
    /**
     * Returns how method parameters should be output in the class diagrams.
     * @returns How method parameters should be output in the class diagrams.
     */
    get methodParameterOutput(): RequiredPluginConfig["methodParameterOutput"];
    /**
     * Returns how class members should be ordered in the class diagrams.
     * @returns How class members should be ordered the class diagrams.
     */
    get memberOrder(): RequiredPluginConfig["memberOrder"];
    /**
     * Returns the boundary before swiching from top->down to left->right direction for class diagrams.
     * @returns The boundary before swiching from top->down to left->right direction for class diagrams.
     */
    get topDownLayoutMaxSiblings(): RequiredPluginConfig["topDownLayoutMaxSiblings"];
    /**
     * Returns how the member visibility is rendered in the class diagrams.
     * @returns How the member visibility is rendered in the class diagrams.
     */
    get visibilityStyle(): RequiredPluginConfig["visibilityStyle"];
    /**
     * Returns whether to hide empty properties and methods in the class diagrams.
     * @returns Whether to hide empty properties and methods in the class diagrams.
     */
    get hideEmptyMembers(): RequiredPluginConfig["hideEmptyMembers"];
    /**
     * Returns whether to hide the circled character in front of class names for class diagrams.
     * @returns Whether to hide the circled character in front of class names for class diagrams.
     */
    get hideCircledChar(): RequiredPluginConfig["hideCircledChar"];
    /**
     * Returns whether to hide the shadows in the class diagrams.
     * @returns Whether to hide the shadows in the class diagrams.
     */
    get hideShadow(): RequiredPluginConfig["hideShadow"];
    /**
     * Returns the color that should be used for the background of all class diagrams.
     * @returns The color that should be used for the background of all class diagrams.
     */
    get backgroundColor(): RequiredPluginConfig["style"]["backgroundColor"];
    /**
     * Returns the style that should be used for boxes in the class diagrams.
     * @returns The style that should be used for boxes in the class diagrams.
     */
    get box(): RequiredPluginConfig["style"]["box"];
    /**
     * Returns the style that should be used for arrows in the class diagrams.
     * @returns The style that should be used for arrows in the class diagrams.
     */
    get arrow(): RequiredPluginConfig["style"]["arrow"];
    /**
     * Returns the style that should be used for classes in the class diagrams.
     * @returns The style that should be used for classes in the class diagrams.
     */
    get class(): RequiredPluginConfig["style"]["class"];
    /**
     * Returns the style that should be used for interfaces in the class diagrams.
     * @returns The style that should be used for interfaces in the class diagrams.
     */
    get interface(): RequiredPluginConfig["style"]["interface"];
    /**
     * Returns the style that should be used for properties in the class diagrams.
     * @returns The style that should be used for properties in the class diagrams.
     */
    get property(): RequiredPluginConfig["style"]["property"];
    /**
     * Returns the style that should be used for properties in the class diagrams.
     * @returns The style that should be used for properties in the class diagrams.
     */
    get method(): RequiredPluginConfig["style"]["method"];
    /**
     * Returns the number of PlantUML processes that should be used for generating the class diagrams.
     * @returns The number of PlantUML processes that should be used for generating the class diagrams.
     */
    get generatorProcessCount(): RequiredPluginConfig["generatorProcessCount"];
    /**
     * Returns whether to hide the progress bar while generating the class diagrams.
     * @returns True, if the progress bar should not be displayed while generating the class diagrams, otherwise false.
     */
    get hideProgressBar(): RequiredPluginConfig["hideProgressBar"];
    /**
     * Returns whether to create files containing the PlantUML code for the class diagrams.
     * @returns True, if files with the PlantUML code should be created, otherwise false.
     */
    get createPlantUmlFiles(): RequiredPluginConfig["createPlantUmlFiles"];
    /**
     * Returns whether the plugin should output verbose info during its work.
     * @returns True, if plugin should create verbose output, otherwise false.
     */
    get createVerboseOutput(): RequiredPluginConfig["verboseOutput"];
    /**
     * Returns whether any text should be styled with an italic font.
     * @returns True if anything should be styled with an italic font, otherwise false.
     */
    get isAnythingStyledItalic(): boolean;
    /**
     * Adds the command line options of the plugin to the TypeDoc application.
     * @param typedoc The TypeDoc application.
     */
    addToApplication(typedoc: Application): void;
    /**
     * Reads the values of the plugin options from the application options.
     * @param typedoc The TypeDoc application.
     */
    readValuesFromApplication(typedoc: Readonly<Application>): void;
}
export {};
