import {} from "./page_section.js";
/**
 * Helper class for finding sections on a TypeDoc page.
 */
// eslint-disable-next-line @typescript-eslint/no-extraneous-class -- TODO: Turn this into a singleton class.
export class PageSectionFinder {
    /**
     * Checks if there is a given section in the content.
     * @param content The content in which to look for the section.
     * @param section The section that is looked for.
     * @returns True, if the section is found in the content, otherwise false.
     */
    static hasSection(content, section) {
        const location = PageSectionFinder.getSectionLocation(content, section);
        return location.startIndex !== location.endIndex;
    }
    /**
     * Returns the position of the section within the content.
     * @param content The content in which to look for the section.
     * @param section The section whos location is wanted.
     * @returns An object with the start and end index of the section.
     *          If the section cannot be found in the content both startIndex and endIndex equal 0.
     */
    static getSectionLocation(content, section) {
        const regex = PageSectionFinder.getLookUpRegexpForSection(section);
        const match = regex.exec(content);
        return match
            ? { startIndex: match.index, endIndex: match.index + match[0].length }
            : { startIndex: 0, endIndex: 0 };
    }
    /**
     * Returns a regular expression that can be used to match a section.
     * @param section The section to match.
     * @returns The regular expression to match the section.
     */
    static getLookUpRegexpForSection(section) {
        let regex = PageSectionFinder.pageSectionLookUpCache.get(section);
        if (!regex) {
            regex = PageSectionFinder.createLookUpRegexpForSection(section);
            PageSectionFinder.pageSectionLookUpCache.set(section, regex);
        }
        return regex;
    }
    /**
     * Creates a regular expression that can be used to match a section with a given name.
     * @param sectionName The name of the section.
     * @returns The regular expression to match the section.
     */
    static createLookUpRegexpForSection(sectionName) {
        return new RegExp('<section class="tsd-panel[^<>]*?">\\s*<h(\\d)>' + sectionName + ".*</h\\1>.*?</section>");
    }
}
/** Map for caching regular expressions to find a section on a page. */
PageSectionFinder.pageSectionLookUpCache = new Map();
